@@ -10,6 +10,7 @@ use {
         launchable::Launchable,
         path::closest_dir,
         skin::*,
+        stage::Stage,
         task_sync::{Dam, Either},
         verb::Internal,
     },
@@ -22,13 +23,12 @@ use {
     std::{
         io::Write,
         path::PathBuf,
+        sync::{Arc, Mutex},
     },
     strict::NonEmptyVec,
     termimad::{Event, EventSource},
 };
 
-#[cfg(feature = "client-server")]
-use std::sync::{Arc, Mutex};
 
 /// The GUI
 pub struct App {
@@ -55,9 +55,8 @@ pub struct App {
 
     stage_panel: Option<PanelId>,
 
-    /// the root of the active panel
-    #[cfg(feature = "client-server")]
-    root: Arc<Mutex<PathBuf>>,
+    /// an optional copy of the root for the --server
+    shared_root: Option<Arc<Mutex<PathBuf>>>,
 
     /// sender to the sequence channel
     tx_seqs: Sender<Sequence>,
@@ -97,9 +96,7 @@ impl App {
             created_panels_count: 1,
             preview_panel: None,
             stage_panel: None,
-
-            #[cfg(feature = "client-server")]
-            root: Arc::new(Mutex::new(con.launch_args.root.clone())),
+            shared_root: None,
             tx_seqs,
             rx_seqs,
         })
@@ -236,7 +233,6 @@ impl App {
 
     /// if there are exactly two non preview panels, return the selection
     /// in the non focused panel
-    /// FIXME exclude stage panel
     fn get_other_panel_path(&self) -> Option<PathBuf> {
         let len = self.panels.len().get();
         if len == 3 {
@@ -271,7 +267,6 @@ impl App {
         let mut error: Option<String> = None;
         let is_input_invocation = cmd.is_verb_invocated_from_input();
         let app_cmd_context = AppCmdContext {
-            other_path: self.get_other_panel_path(),
             panel_skin,
             preview_panel: self.preview_panel,
             stage_panel: self.stage_panel,
@@ -325,7 +320,6 @@ impl App {
                         let new_input = self.panel().get_input_content();
                         let cmd = Command::from_raw(new_input, false);
                         let app_cmd_context = AppCmdContext {
-                            other_path: self.get_other_panel_path(),
                             panel_skin,
                             preview_panel: self.preview_panel,
                             stage_panel: self.stage_panel,
@@ -373,7 +367,6 @@ impl App {
                             }
                             self.active_panel_idx = idx;
                             let app_cmd_context = AppCmdContext {
-                                other_path: self.get_other_panel_path(),
                                 panel_skin,
                                 preview_panel: self.preview_panel,
                                 stage_panel: self.stage_panel,
@@ -476,7 +469,6 @@ impl App {
                 }
                 if self.remove_state() {
                     let app_cmd_context = AppCmdContext {
-                        other_path: self.get_other_panel_path(),
                         panel_skin,
                         preview_panel: self.preview_panel,
                         stage_panel: self.stage_panel,
@@ -507,13 +499,21 @@ impl App {
         if let Some(text) = error {
             self.mut_panel().set_error(text);
         }
-        self.update_preview(con);
 
-        #[cfg(feature="client-server")]
-        if let Ok(mut root) = self.root.lock() { // when does this not work ?
-            *root = self.state().selected_path().to_path_buf();
+        app_state.other_panel_path = self.get_other_panel_path();
+        if let Some(path) = self.state().tree_root() {
+            app_state.root = path.to_path_buf();
         }
 
+
+        if let Some(shared_root) = &mut self.shared_root {
+            if let Ok(mut root) = shared_root.lock() {
+                *root = app_state.root.clone();
+            }
+        }
+
+        self.update_preview(con);
+
         Ok(())
     }
 
@@ -610,7 +610,6 @@ impl App {
             if self.do_pending_task(app_state, con, dam) {
                 self.update_preview(con); // the selection may have changed
                 let app_cmd_context = AppCmdContext {
-                    other_path: self.get_other_panel_path(),
                     panel_skin: &skin.focused,
                     preview_panel: self.preview_panel,
                     stage_panel: self.stage_panel,
@@ -618,7 +617,7 @@ impl App {
                     con,
                 };
                 self.mut_panel().refresh_input_status(app_state, &app_cmd_context);
-                self.display_panels(w, &skin, app_state, con)?;
+                self.display_panels(w, skin, app_state, con)?;
             } else {
                 warn!("unexpected lack of update on do_pending_task");
                 return Ok(());
@@ -669,8 +668,12 @@ impl App {
         let event_source = EventSource::new()?;
         let rx_events = event_source.receiver();
         let mut dam = Dam::from(rx_events);
-        let skin = AppSkin::new(conf, con.launch_args.no_style);
-        let mut app_state = AppState::default();
+        let skin = AppSkin::new(conf, con.launch_args.color == Some(false));
+        let mut app_state = AppState {
+            stage: Stage::default(),
+            root: con.launch_args.root.clone(),
+            other_panel_path: None,
+        };
 
         self.screen.clear_bottom_right_char(w, &skin.focused)?;
 
@@ -680,13 +683,18 @@ impl App {
                 .unwrap();
         }
 
-        #[cfg(feature="client-server")]
+        #[cfg(unix)]
         let _server = con.launch_args.listen.as_ref()
-            .map(|server_name| crate::net::Server::new(
-                &server_name,
-                self.tx_seqs.clone(),
-                Arc::clone(&self.root),
-            ))
+            .map(|server_name| {
+                let shared_root = Arc::new(Mutex::new(app_state.root.clone()));
+                let server = crate::net::Server::new(
+                    server_name,
+                    self.tx_seqs.clone(),
+                    Arc::clone(&shared_root),
+                );
+                self.shared_root = Some(shared_root);
+                server
+            })
             .transpose()?;
 
         loop {
