@@ -60,7 +60,7 @@ impl FuzzyPattern {
             2 => 1,
             3 => 2,
             4 => 2,
-            5 => 3,
+            5 => 2,
             6 => 3,
             7 => 3,
             8 => 4,
@@ -99,6 +99,7 @@ impl FuzzyPattern {
                             break;
                         }
                         if cand_chars[cand_idx-rev_idx] == self.chars[pat_idx-rev_idx] {
+                            // we move the pos forward
                             pos[pat_idx-rev_idx] = cand_idx-rev_idx;
                         } else {
                             break;
@@ -126,10 +127,22 @@ impl FuzzyPattern {
             if pos[idx] > 1 + pos[idx-1] {
                 nb_holes += 1;
                 if idx > 1 && pos[idx-1] > 1 + pos[idx-2] {
-                    nb_singled_chars += 1;
+                    // we improve a simple case: the one of a singleton which was created
+                    // by pushing forward a char
+                    if cand_chars[pos[idx-2]+1] == cand_chars[pos[idx-1]] {
+                        // in some cases we're really removing another singletons but
+                        // let's forget this
+                        pos[idx-1] = pos[idx-2]+1;
+                        nb_holes -= 1;
+                    } else {
+                        nb_singled_chars += 1;
+                    }
                 }
             }
         }
+        if nb_holes > self.max_nb_holes {
+            return MatchSearchResult::None;
+        }
         let match_len = 1 + cand_idx - pos[0];
         let mut score = BONUS_MATCH;
         score += BONUS_CANDIDATE_LENGTH * (cand_chars.len() as i32);
@@ -267,6 +280,11 @@ mod fuzzy_pattern_tests {
             "brbroorrbbbbbrrooorobrototooooot.txt",
             "                    ^^^ ^^          ",
         );
+        check_pos(
+            "besh",
+            "benches/shared",
+            "^^      ^^    ",
+        );
     }
 
     /// check that the scores of all names are strictly decreasing
